package body QUCS.Datasets is


   ------------------------
   -- Basic_Add_Variable --
   ------------------------

   procedure Basic_Add_Variable
     (Container : in out Dataset;
      Name      : String;
      Var       : Variables.Variable)
   is
   begin
      if Container.Exist (Name) then
         raise Constraint_Error;
      end if;

      declare
         Descr : Var_Descriptor := (Name         => To_Unbounded_String (Name),
                                    Var          => Var,
                                    Dependencies => Name_Lists.Empty_Vector);
      begin
         Container.Table.Insert (Key      => Name,
                                 New_Item => Descr);
      end;

      Container.Addiction_History.Append (Name);
   end Basic_Add_Variable;

   ------------------
   -- Add_Variable --
   ------------------

   procedure Add_Variable
     (Container : in out Dataset;
      Name      : String;
      Size      : Positive)
   is
   begin
      Basic_Add_Variable (Container,
                          Name,
                          Variables.Create (Size));
   end Add_Variable;


   ------------------
   -- Add_Variable --
   ------------------

   procedure Add_Variable
     (Container : in out Dataset;
      Name      : String;
      Size      : Variables.Dim_Array)
   is
   begin
      Basic_Add_Variable (Container,
                          Name,
                          Variables.Create (Size));
   end Add_Variable;


   -----------
   -- Exist --
   -----------

   function Exist
     (Container : Dataset;
      Name      : String)
      return Boolean
   is
   begin
      return Container.Table.Contains (Name);
   end Exist;

   ---------
   -- Get --
   ---------

   function Get
     (Container : Dataset;
      Name      : String)
      return QUCS.Variables.Variable
   is
   begin
      if not Container.Exist (Name) then
         raise Constraint_Error;
      end if;

      return Container.Table.Element (Name).Var;
   end Get;

   ---------
   -- Get --
   ---------

   function Get
     (Container : Dataset;
      Name      : String;
      Index     : Positive)
      return Float
   is
   begin
      if not Container.Exist (Name) then
         raise Constraint_Error;
      end if;

      return Container.Table.Element (Name).Var.Get (Index);
   end Get;

   ---------
   -- Get --
   ---------

   function Get
     (Position : Cursor)
      return QUCS.Variables.Variable
   is
   begin
      return Var_Tables.Element (Var_Tables.Cursor (Position)).Var;
   end Get;

   ---------
   -- Get --
   ---------

   function Get
     (Position : Cursor;
      Index    : Positive)
      return Float
   is
   begin
      return Var_Tables.Element (Var_Tables.Cursor (Position)).Var.Get (Index);
   end Get;

   ----------
   -- Name --
   ----------

   function Name (Position : Cursor) return String is
   begin
      return To_String (Var_Tables.Element (Var_Tables.Cursor (Position)).Name);
   end Name;

   --------------------
   -- Add_Dependency --
   --------------------

   procedure Add_Dependency
     (Container  : in out Dataset;
      Name       : String;
      Depends_On : String)
   is
      procedure Update (Key  : String;
                        Item : in out Var_Descriptor) is
      begin
         Item.Dependencies.Append (Depends_On);
      end Update;
   begin
      if not Container.Exist (Name) then
         raise Constraint_Error;
      end if;

      Container.Table.Update_Element (Position => Container.Table.Find (Name),
                                      Process  => Update'Access);
   end Add_Dependency;

   ------------------
   -- Dependencies --
   ------------------

   function Dependencies
     (Container : Dataset;
      Name      : String)
      return Name_Lists.Vector
   is
   begin
      if not Container.Exist (Name) then
         raise Constraint_Error;
      end if;

      return Container.Table.Element (Name).Dependencies;
   end Dependencies;

   ------------------
   -- Dependencies --
   ------------------

   function Dependencies
     (Position : Cursor)
      return Name_Lists.Vector
   is
   begin
      return Var_Tables.Element (Var_Tables.Cursor (Position)).Dependencies;
   end Dependencies;

   ---------
   -- Set --
   ---------

   procedure Set
     (Container : in out Dataset;
      Name      : String;
      Index     : Positive;
      Value     : Float)
   is
      procedure Update (Key  : String;
                        Item : in out Var_Descriptor) is
      begin
         Item.Var.Set (Index, Value);
      end Update;
   begin
      if not Container.Exist (Name) then
         raise Constraint_Error;
      end if;

      Container.Table.Update_Element (Position => Container.Table.Find (Name),
                                      Process  => Update'Access);
   end Set;

   -------------
   -- Iterate --
   -------------

   procedure Iterate
     (Container : Dataset;
      Callback  : access procedure (Position : Cursor))
   is
      procedure Gateway (Position : Name_Lists.Cursor) is
         Key : constant String := Name_Lists.Element(Position);
      begin
         Callback (Cursor(Container.Table.Find(Key)));
      end Gateway;
   begin
      Container.Addiction_History.Iterate (Gateway'Access);
   end Iterate;

end QUCS.Datasets;
