with Ada.Containers.Indefinite_Vectors;
with Ada.Strings.Unbounded;
with Ada.Containers.Indefinite_Ordered_Maps;

with QUCS.Variables;

package QUCS.Datasets is
   package Name_Lists is
     new Ada.Containers.Indefinite_Vectors (Index_Type   => Positive,
                                            Element_Type => String);

   type Dataset is tagged private;

   type Cursor is private;

   No_Element : constant Cursor;

   procedure Add_Variable
     (Container : in out Dataset;
      Name      : String;
      Size      : Positive);

   procedure Add_Variable
     (Container : in out Dataset;
      Name      : String;
      Size      : Variables.Dim_Array);


   function Exist (Container : Dataset;
                   Name      : String)
                   return Boolean;

   function Get (Container : Dataset;
                 Name      : String)
                 return QUCS.Variables.Variable;

   function Get (Container : Dataset;
                 Name      : String;
                 Index     : Positive)
                 return Float;

   function Get (Position : Cursor)
                 return QUCS.Variables.Variable;

   function Get (Position : Cursor;
                 Index    : Positive)
                 return Float;

   function Name (Position : Cursor) return String;


   procedure Add_Dependency
     (Container  : in out Dataset;
      Name       : String;
      Depends_On : String);

   function Dependencies (Container : Dataset;
                          Name      : String)
                          return Name_Lists.Vector;

   function Dependencies (Position : Cursor)
                          return Name_Lists.Vector;


   procedure Set (Container : in out Dataset;
                  Name      : String;
                  Index     : Positive;
                  Value     : Float);

   procedure Iterate (Container : Dataset;
                      Callback  : access procedure (Position : Cursor));
private
   use Ada.Strings.Unbounded;

   type Var_Descriptor is
      record
         Name : Unbounded_String;
         Var  : Variables.Variable;
         Dependencies : Name_Lists.Vector;
      end record;

   package Var_Tables is
     new Ada.Containers.Indefinite_Ordered_Maps
       (Key_Type     => String,
        Element_Type => Var_Descriptor);

   type Dataset is tagged
      record
         Table : Var_Tables.Map;
         Addiction_History : Name_Lists.Vector;
      end record;

   type Cursor is new Var_Tables.Cursor;

   No_Element : constant Cursor := Cursor (Var_Tables.No_Element);
end QUCS.Datasets;

