with Ada.Containers.Vectors;

package QUCS.Variables is
   type Dim_Array is array (Positive range <>) of Positive;

   type Variable is tagged private;

   function Create (Size : Positive) return Variable;

   function Create (Size : Dim_Array) return Variable;

--     procedure Append (Item  : in out Variable;
--                       Value : Float);

   function Size (Item : Variable) return Positive;

   function Size (Item : Variable) return Dim_Array;

   procedure Set (Item  : in out Variable;
                  Index : Positive;
                  Value : Float);

   function Get (Item  : Variable;
                 Index : Positive)
                 return Float;

   function Total_Size (Item  : Variable) return Positive;

   generic
      type Float_Type is  digits <>;
      type Float_Array is array (Positive range <>) of Float_Type;
   function To_Array (Item : Variable)
                      return Float_Array;

private
   subtype Dimensionality is Integer range 1 .. 64;

   type Dimension_Array is array (Dimensionality range <>) of Positive;

   package Float_Vectors is
     new Ada.Containers.Vectors (Index_Type   => Positive,
                                 Element_Type => Float);

   type Variable is tagged
      record
         N_Dims : Dimensionality;
         Dims   : Dimension_Array (Dimensionality);
         Data : Float_Vectors.Vector;
      end record;
end QUCS.Variables;
